package java.net;

import java.io.InvalidObjectException;
import java.io.ObjectStreamException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.StringTokenizer;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.io.InputStream;
import sun.net.ApplicationProxy;
import java.io.IOException;
import sun.security.util.SecurityConstants;
import sun.net.www.protocol.jar.Handler;
import sun.net.util.IPAddressUtil;
import java.io.ObjectStreamField;
import java.util.Hashtable;
import java.io.Serializable;

public final class URL implements Serializable
{
    static final String BUILTIN_HANDLERS_PREFIX = "sun.net.www.protocol";
    static final long serialVersionUID = -7627629688361524110L;
    private static final String protocolPathProp = "java.protocol.handler.pkgs";
    private String protocol;
    private String host;
    private int port;
    private String file;
    private transient String query;
    private String authority;
    private transient String path;
    private transient String userInfo;
    private String ref;
    transient InetAddress hostAddress;
    transient URLStreamHandler handler;
    private int hashCode;
    private transient UrlDeserializedState tempState;
    static URLStreamHandlerFactory factory;
    static Hashtable<String, URLStreamHandler> handlers;
    private static Object streamHandlerLock;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public URL(String s, String s2, int n, String s3) throws MalformedURLException {
        this(s, s2, n, s3, null);
    }
    
    public URL(String s, String s2, String s3) throws MalformedURLException {
        this(s, s2, -1, s3);
    }
    
    public URL(String lowerCase, String string, int i, String spec, URLStreamHandler urlStreamHandler) throws MalformedURLException {
        this.port = -1;
        this.hashCode = -1;
        if (urlStreamHandler != null) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                this.checkSpecifyHandler(securityManager);
            }
        }
        lowerCase = lowerCase.toLowerCase();
        this.protocol = lowerCase;
        if (string != null) {
            if (string.indexOf(58) >= 0 && !string.startsWith("[")) {
                string = "[" + string + "]";
            }
            this.host = string;
            if (i < -1) {
                throw new MalformedURLException("Invalid port number :" + i);
            }
            this.authority = (((this.port = i) == -1) ? string : (string + ":" + i));
        }
        Parts o = new Parts(spec);
        this.path = o.getPath();
        this.query = o.getQuery();
        if (this.query != null) {
            this.file = this.path + "?" + this.query;
        }
        else {
            this.file = this.path;
        }
        this.ref = o.getRef();
        if (urlStreamHandler == null && (urlStreamHandler = getURLStreamHandler(lowerCase)) == null) {
            throw new MalformedURLException("unknown protocol: " + lowerCase);
        }
        this.handler = urlStreamHandler;
        if (string != null && this.isBuiltinStreamHandler(urlStreamHandler)) {
            String checkExternalForm = IPAddressUtil.checkExternalForm(this);
            if (checkExternalForm != null) {
                throw new MalformedURLException(checkExternalForm);
            }
        }
        if ("jar".equalsIgnoreCase(lowerCase) && urlStreamHandler instanceof Handler) {
            String checkNestedProtocol = ((Handler)urlStreamHandler).checkNestedProtocol(spec);
            if (checkNestedProtocol != null) {
                throw new MalformedURLException(checkNestedProtocol);
            }
        }
        return;
    }
    
    public URL(String s) throws MalformedURLException {
        this(null, s);
    }
    
    public URL(URL url, String s) throws MalformedURLException {
        this(url, s, null);
    }
    
    public URL(URL url, String str, URLStreamHandler handler) throws MalformedURLException {
        this.port = -1;
        this.hashCode = -1;
        int fromIndex = 0;
        String protocol = null;
        boolean n = false;
        boolean n2 = false;
        if (handler != null) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                this.checkSpecifyHandler(securityManager);
            }
        }
        try {
            int length;
            for (length = str.length(); length > 0 && str.charAt(length - 1) <= ' '; --length) {}
            while (fromIndex < length && str.charAt(fromIndex) <= ' ') {
                ++fromIndex;
            }
            if (str.regionMatches(true, fromIndex, "url:", 0, 4)) {
                fromIndex += 4;
            }
            if (fromIndex < str.length() && str.charAt(fromIndex) == '#') {
                n = true;
            }
            int char1;
            while (n == 0 && fromIndex < length && (char1 = str.charAt(fromIndex)) != '/') {
                if (char1 == ':') {
                    String lowerCase = str.substring(fromIndex, fromIndex).toLowerCase();
                    if (this.isValidProtocol(lowerCase)) {
                        protocol = lowerCase;
                        ++fromIndex;
                        break;
                    }
                    break;
                }
                else {
                    ++fromIndex;
                }
            }
            this.protocol = protocol;
            if (url != null && (protocol == null || protocol.equalsIgnoreCase(url.protocol))) {
                if (handler == null) {
                    handler = url.handler;
                }
                if (url.path != null && url.path.startsWith("/")) {
                    protocol = null;
                }
                if (protocol == null) {
                    this.protocol = url.protocol;
                    this.authority = url.authority;
                    this.userInfo = url.userInfo;
                    this.host = url.host;
                    this.port = url.port;
                    this.file = url.file;
                    this.path = url.path;
                    n2 = true;
                }
            }
            if (this.protocol == null) {
                throw new MalformedURLException("no protocol: " + str);
            }
            if (handler == null && (handler = getURLStreamHandler(this.protocol)) == null) {
                throw new MalformedURLException("unknown protocol: " + this.protocol);
            }
            this.handler = handler;
            int index = str.indexOf(35, fromIndex);
            if (index >= 0) {
                this.ref = str.substring(index + 1, length);
                length = index;
            }
            if (n2 && fromIndex == length) {
                this.query = url.query;
                if (this.ref == null) {
                    this.ref = url.ref;
                }
            }
            handler.parseURL(this, str, fromIndex, length);
        }
        catch (MalformedURLException ex) {
            throw ex;
        }
        catch (Exception cause) {
            MalformedURLException ex2 = new MalformedURLException(cause.getMessage());
            ex2.initCause(cause);
            throw ex2;
        }
    }
    
    private boolean isValidProtocol(String s) {
        int length = s.length();
        if (length < 1) {
            return false;
        }
        if (!Character.isLetter(s.charAt(0))) {
            return false;
        }
        int i = 1;
        while (i < length) {
            int char1 = s.charAt(i);
            if (!Character.isLetterOrDigit((char)char1) && char1 != '.' && char1 != '+' && char1 != '-') {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private void checkSpecifyHandler(SecurityManager securityManager) {
        securityManager.checkPermission(SecurityConstants.SPECIFY_HANDLER_PERMISSION);
    }
    
    void set(String protocol, String s, int n, String s2, String ref) {
        synchronized (this) {
            this.protocol = protocol;
            this.host = s;
            this.authority = ((n == -1) ? s : (s + ":" + n));
            this.port = n;
            this.file = s2;
            this.ref = ref;
            this.hashCode = -1;
            this.hostAddress = null;
            int lastIndex = s2.lastIndexOf(63);
            if (lastIndex != -1) {
                this.query = s2.substring(lastIndex + 1);
                this.path = s2.substring(0, lastIndex);
            }
            else {
                this.path = s2;
            }
        }
    }
    
    void set(String protocol, String host, int port, String authority, String userInfo, String s, String s2, String ref) {
        synchronized (this) {
            this.protocol = protocol;
            this.host = host;
            this.port = port;
            this.file = ((s2 == null) ? s : (s + "?" + s2));
            this.userInfo = userInfo;
            this.path = s;
            this.ref = ref;
            this.hashCode = -1;
            this.hostAddress = null;
            this.query = s2;
            this.authority = authority;
        }
    }
    
    public String getQuery() {
        return this.query;
    }
    
    public String getPath() {
        return this.path;
    }
    
    public String getUserInfo() {
        return this.userInfo;
    }
    
    public String getAuthority() {
        return this.authority;
    }
    
    public int getPort() {
        return this.port;
    }
    
    public int getDefaultPort() {
        return this.handler.getDefaultPort();
    }
    
    public String getProtocol() {
        return this.protocol;
    }
    
    public String getHost() {
        return this.host;
    }
    
    public String getFile() {
        return this.file;
    }
    
    public String getRef() {
        return this.ref;
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof URL && this.handler.equals(this, (URL)o);
    }
    
    @Override
    public synchronized int hashCode() {
        if (this.hashCode != -1) {
            return this.hashCode;
        }
        return this.hashCode = this.handler.hashCode(this);
    }
    
    public boolean sameFile(URL url) {
        return this.handler.sameFile(this, url);
    }
    
    @Override
    public String toString() {
        return this.toExternalForm();
    }
    
    public String toExternalForm() {
        return this.handler.toExternalForm(this);
    }
    
    public URI toURI() throws URISyntaxException {
        URI uri = new URI(this.toString());
        if (this.authority != null && this.isBuiltinStreamHandler(this.handler)) {
            String checkAuthority = IPAddressUtil.checkAuthority(this);
            if (checkAuthority != null) {
                throw new URISyntaxException(this.authority, checkAuthority);
            }
        }
        return uri;
    }
    
    public URLConnection openConnection() throws IOException {
        return this.handler.openConnection(this);
    }
    
    public URLConnection openConnection(Proxy proxy) throws IOException {
        if (proxy == null) {
            throw new IllegalArgumentException("proxy can not be null");
        }
        Proxy proxy2 = (proxy == Proxy.NO_PROXY) ? Proxy.NO_PROXY : ApplicationProxy.create(proxy);
        SecurityManager securityManager = System.getSecurityManager();
        if (proxy2.type() != Proxy.Type.DIRECT && securityManager != null) {
            InetSocketAddress inetSocketAddress = (InetSocketAddress)proxy2.address();
            if (inetSocketAddress.isUnresolved()) {
                securityManager.checkConnect(inetSocketAddress.getHostName(), inetSocketAddress.getPort());
            }
            else {
                securityManager.checkConnect(inetSocketAddress.getAddress().getHostAddress(), inetSocketAddress.getPort());
            }
        }
        return this.handler.openConnection(this, proxy2);
    }
    
    public final InputStream openStream() throws IOException {
        return this.openConnection().getInputStream();
    }
    
    public final Object getContent() throws IOException {
        return this.openConnection().getContent();
    }
    
    public final Object getContent(Class[] array) throws IOException {
        return this.openConnection().getContent(array);
    }
    
    public static void setURLStreamHandlerFactory(URLStreamHandlerFactory factory) {
        synchronized (URL.streamHandlerLock) {
            if (URL.factory != null) {
                throw new Error("factory already defined");
            }
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkSetFactory();
            }
            URL.handlers.clear();
            URL.factory = factory;
        }
    }
    
    static URLStreamHandler getURLStreamHandler(String s) {
        URLStreamHandler urlStreamHandler = (URLStreamHandler)URL.handlers.get(s);
        if (urlStreamHandler == null) {
            boolean n = false;
            if (URL.factory != null) {
                urlStreamHandler = URL.factory.createURLStreamHandler(s);
                n = true;
            }
            if (urlStreamHandler == null) {
                String string = (String)AccessController.doPrivileged(new GetPropertyAction("java.protocol.handler.pkgs", ""));
                if (string != "") {
                    string += "|";
                }
                StringTokenizer o = new StringTokenizer(string + "sun.net.www.protocol", "|");
                while (urlStreamHandler == null && o.hasMoreTokens()) {
                    String trim = o.nextToken().trim();
                    try {
                        String string2 = trim + "." + s + ".Handler";
                        Class<Object> clazz = null;
                        try {
                            clazz = Class.forName(string2);
                        }
                        catch (ClassNotFoundException ex) {
                            ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
                            if (systemClassLoader != null) {
                                clazz = systemClassLoader.loadClass(string2);
                            }
                        }
                        if (clazz != null) {
                            urlStreamHandler = (URLStreamHandler)clazz.newInstance();
                        }
                        else {
                            continue;
                        }
                    }
                    catch (Exception ex2) {}
                }
            }
            synchronized (URL.streamHandlerLock) {
                URLStreamHandler urlStreamHandler2 = (URLStreamHandler)URL.handlers.get(s);
                if (urlStreamHandler2 != null) {
                    return urlStreamHandler2;
                }
                if (n == 0 && URL.factory != null) {
                    urlStreamHandler2 = URL.factory.createURLStreamHandler(s);
                }
                if (urlStreamHandler2 != null) {
                    urlStreamHandler = urlStreamHandler2;
                }
                if (urlStreamHandler != null) {
                    URL.handlers.put(s, urlStreamHandler);
                }
            }
        }
        return urlStreamHandler;
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
    }
    
    private synchronized void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        String str = (String)fields.get("protocol", null);
        if (getURLStreamHandler(str) == null) {
            throw new IOException("unknown protocol: " + str);
        }
        String str2 = (String)fields.get("host", null);
        int value = fields.get("port", -1);
        String s = (String)fields.get("authority", null);
        String s2 = (String)fields.get("file", null);
        String s3 = (String)fields.get("ref", null);
        int value2 = fields.get("hashCode", -1);
        if (s == null && ((str2 != null && str2.length() > 0) || value != -1)) {
            if (str2 == null) {
                str2 = "";
            }
            s = ((value == -1) ? str2 : (str2 + ":" + value));
        }
        this.tempState = new UrlDeserializedState(str, str2, value, s, s2, s3, value2);
        return;
    }
    
    private Object readResolve() throws ObjectStreamException {
        URLStreamHandler urlStreamHandler = getURLStreamHandler(this.tempState.getProtocol());
        if (this.isBuiltinStreamHandler(urlStreamHandler.getClass().getName())) {
            return this.fabricateNewURL();
        }
        return this.setDeserializedFields(urlStreamHandler);
    }
    
    private URL setDeserializedFields(URLStreamHandler handler) {
        String userInfo = null;
        String protocol = this.tempState.getProtocol();
        String s = this.tempState.getHost();
        int port = this.tempState.getPort();
        String authority = this.tempState.getAuthority();
        String file = this.tempState.getFile();
        String ref = this.tempState.getRef();
        int hashCode = this.tempState.getHashCode();
        if (authority == null && ((s != null && s.length() > 0) || port != -1)) {
            if (s == null) {
                s = "";
            }
            authority = ((port == -1) ? s : (s + ":" + port));
            int lastIndex = s.lastIndexOf(64);
            if (lastIndex != -1) {
                userInfo = s.substring(0, lastIndex);
                s = s.substring(lastIndex + 1);
            }
        }
        else if (authority != null) {
            int index = authority.indexOf(64);
            if (index != -1) {
                userInfo = authority.substring(0, index);
            }
        }
        String substring = null;
        String substring2 = null;
        if (file != null) {
            int lastIndex2 = file.lastIndexOf(63);
            if (lastIndex2 != -1) {
                substring2 = file.substring(lastIndex2 + 1);
                substring = file.substring(0, lastIndex2);
            }
            else {
                substring = file;
            }
        }
        this.protocol = protocol;
        this.host = s;
        this.port = port;
        this.file = file;
        this.authority = authority;
        this.ref = ref;
        this.hashCode = hashCode;
        this.handler = handler;
        this.query = substring2;
        this.path = substring;
        this.userInfo = userInfo;
        return this;
    }
    
    private URL fabricateNewURL() throws InvalidObjectException {
        String reconstituteUrlString = this.tempState.reconstituteUrlString();
        URL url;
        try {
            url = new URL(reconstituteUrlString);
        }
        catch (MalformedURLException cause) {
            this.resetState();
            InvalidObjectException ex = new InvalidObjectException("Malformed URL: " + reconstituteUrlString);
            ex.initCause(cause);
            throw ex;
        }
        url.setSerializedHashCode(this.tempState.getHashCode());
        this.resetState();
        return url;
    }
    
    boolean isBuiltinStreamHandler(URLStreamHandler urlStreamHandler) {
        Class<Object> class1 = urlStreamHandler.getClass();
        return this.isBuiltinStreamHandler(class1.getName()) || class1.getClassLoader() == null;
    }
    
    private boolean isBuiltinStreamHandler(String s) {
        return s.startsWith("sun.net.www.protocol");
    }
    
    private void resetState() {
        this.protocol = null;
        this.host = null;
        this.port = -1;
        this.file = null;
        this.authority = null;
        this.ref = null;
        this.hashCode = -1;
        this.handler = null;
        this.query = null;
        this.path = null;
        this.userInfo = null;
        this.tempState = null;
    }
    
    private void setSerializedHashCode(int hashCode) {
        this.hashCode = hashCode;
    }
    
    static {
        URL.handlers = new Hashtable();
        URL.streamHandlerLock = new Object();
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("protocol", String.class), new ObjectStreamField("host", String.class), new ObjectStreamField("port", Integer.TYPE), new ObjectStreamField("authority", String.class), new ObjectStreamField("file", String.class), new ObjectStreamField("ref", String.class), new ObjectStreamField("hashCode", Integer.TYPE) };
    }
}
